/* Copyright 1997 Lucent Technologies Inc. - All Rights Reserved */

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include <sys/time.h>

#include "crypto.h"
#include "util.h"

char *
dd(DIGEST d, int n) {
	static char buf[HASHLEN*3+3];
	char buf2[10];
	int i;

	if (n == 0)
		n = HASHLEN;
		
	buf[0] = '\0';
	for (i=0; i<n; i++) {
		sprintf(buf2, "%.02x ", d[i]);
		strcat(buf, buf2);
	}
	return buf;
}

char *
dk(KEY k, int n) {
	static char buf[KEYLEN*3+3];
	char buf2[10];
	int i;

	if (n == 0)
		n = KEYLEN;

	buf[0] = '\0';
	for (i=0; i<n; i++) {
		sprintf(buf2, "%.02x ", k[i]);
		strcat(buf, buf2);
	}
	return buf;
}

char *
ds(u_char *buf, int max, int len) {
	static char obuf[20000];
	char buf2[10];
	int i;

	obuf[0] = '\0';
	if (max < len)
		len = max;

	for (i=0; i<len; i++) {
		if (buf[i] >= ' ' && buf[i] < 0177) {
			char *cp = &obuf[strlen(obuf) + 1];
			*cp++ = buf[i];
			*cp++ = '\0';
		} else {
			sprintf(buf2, "%.02x ", buf[i]);
			strcat(obuf, buf2);
		}
	}
	return obuf;
}

void
cwrites(int fd, char *buf) {
	if (cwrite(fd, buf, strlen(buf)+1) < 0) {
		log("cwrite error: %s", cerror);
		exit(1);
	}
}

/*
 * We require 'n' bytes from the net.   Abort if we don't get them.
 */
int
netread(int fd, char *buf, int n) {
	int i, m=n;

	for (i=0; n>0; n -= i, buf += i) {
		i = read(fd, buf, n);
		if (i == 0)
			return -2;
		if (i < 0)
			return -1;
	}
	return m;
}

int
netwrite(int fd, char *buf, int len) {
	int i, n=len;

	for (i=0; n>0; n -= i, buf += i) {
		i = write(fd, buf, n);
		if (i <= 0)
			return -1;
	}
	return len;
}

/*
 * Find the service name at the beginning of the line in the service
 * file.  If found, return zero, else and error string.  If it is found,
 * strtok will point to the next field.
 */
char *
findservice(char *fn, char *servicename) {
	FILE *f;
	static char buf[MAXIO];
	char *cp;

	f = fopen(fn, "r");
	if (f == 0) {
		sprintf(buf, "can't open config file %s: %s", 
			fn, strerror(errno));
		return buf;
	}

	while (fgets(buf, sizeof(buf), f)) {
		cp = strchr(buf, '#');
		if (cp)
			*cp = '\0';
		if ((cp = strtok(buf, " \t\n")) == 0)
			continue;
		if (strcmp(cp, servicename) == 0) {
			fclose(f);
			return 0;
		}
	}
	fclose(f);
	sprintf(buf, "can't find service '%s' in config file '%s'",
		servicename, fn);
	return buf;
}

void
get_key(char *keyfile, char *service, u_char *key) {
	char buf[100];
	char *ep;

	if (keyfile == 0) {
		strcpy(buf, KEYDIR);
		strcat(buf, "/");
		strncat(buf, service, sizeof(buf) - strlen(service) - 2);
		buf[sizeof(buf)-1] = '\0';
		ep = readkey(buf, key);
	} else
		ep = readkey(keyfile, key);

	if (ep) {
		fprintf(stderr, "%s\n", ep);
		exit(1);
	}
}

u_long
now(void) {
	struct timeval tp;

	if (gettimeofday(&tp, 0) < 0)
		perror("gettimeofday");
	return (((tp.tv_sec)*1000) % 10000) + tp.tv_usec/1000;
}

/* XXX these should probably have timeouts */

/*
 * Copy a file from the network link to a local file.  If we
 * get a local error, continue reading bytes until the sender
 * is finished, because our protocol is stupid, and the sender
 * won't read our error message until he has had his say.
 * This should be rare, which is why we do it this way.
 */
char *
readnetfile(int netfd, int lfd, u_long len) {
	char buf[MAXIO];
	static char err[200];

	strcpy(err, "OK");
	while (len > 0) {
		int i, j;
		int n = len;

		if (n > sizeof(buf))
			n = sizeof(buf);
		i = cread(netfd, buf, n);
		if (i <= 0) {
			if (i == C_EOF || i == 0)
				strcpy(err, "file truncated");
			else
				sprintf(err, "error reading file: %s",
					cerror);
			break;
		}
		j = write(lfd, buf, i);
		if (j <= 0) {
			sprintf(err, "error writing file: %s",
				strerror(errno));
			break;
		}
		len -= i;
	}
	if (cwrite(netfd, err, strlen(err)+1) < 0)
		return cerror;
	if (strcmp(err, "OK") != 0)
		return err;
	return 0;
}

/*
 * Copy a file out to the net connection.  We copy 'len' bytes,
 * which the other end already knows to expect.
 * Send a zero length record if there is an error.
 */
char *
writenetfile(int lfd, int netfd, u_long len) {
	char buf[MAXIO];
	static char err[100];

	while (len > 0) {
		int i, j, k;
		int n = len;

		if (n > sizeof(buf))
			n = sizeof(buf);
		i = read(lfd, buf, n);
		if (i < 0) {
			sprintf(err, "error reading file: %s",
				strerror(errno));
			cwrite(netfd, buf, 0);	/* eof signals error */
			return err;
		}
		if (i == 0) {
			len = 0;
			cwrite(netfd, buf, 0);	/* eof signals error */
			return "source file truncated during transfer";
		}
		if (cwrite(netfd, buf, i) < 0) {
			sprintf(err, "error writing to net: %s", cerror);
			return err;
		}
		len -= i;
	}
	if (cread(netfd, err, sizeof(err)) < 0)
		return "transfer unconfirmed";
	if (strcmp(err, "OK") != 0)
		return err;
	if (read(lfd, buf, 1) > 0)
		return "source file increased during transfer";
	return 0;
}
